package supercars3.game.cars;



import java.awt.Graphics2D;
import java.awt.Rectangle;

import supercars3.base.*;

import supercars3.game.*;
import supercars3.game.players.Driver;
import supercars3.game.weapons.WeaponFactory;


public class ComputerCar extends Car implements RoutableMobile
{
	private final double ANGULAR_SPEED = ro_settings.cpu_angular_speed;
	private MobileGuider m_guider;
	private final int LOCK_AIM_TIME = ro_settings.lock_aim_time;
	
	public void die()
	{
		die(false);
	}
	public void on_gate_bounce()
	{
		m_guider.lock_aim_change(LOCK_AIM_TIME); // no aim change during a while to focus on the gate
	}
	public void on_wall_bounce()
	{
		m_guider.on_wall_bounce();
	}

	 public void killed_by(Car killer_car)
	  {
		 super.killed_by(killer_car);
		 m_guider.reset();
	  }

	 public void race_start(int laps_to_go, CircuitData data, 
				int nb_checkpoints, Route current_route)
	 {
		 super.race_start(laps_to_go, data, nb_checkpoints, current_route);
		
		 m_guider.reset();
	 }
	
	protected void p_render(Graphics2D g, Rectangle view_bounds)
	{
		  super.p_render(g, view_bounds);
		  m_guider.render(g,get_debug_paint());
		  //g.drawString(get_equipment().accessory[Equipment.ENGINE].get_count()+"", (int)m_current.location.x,(int)m_current.location.y);
	}
	
	public void zone_entered(Zone zone,Boundary by)
	{
		 m_guider.zone_entered(zone,this);
		 
		 super.zone_entered(zone, by);
	}
	
	private double angle_variation() 
	{
	    return ANGULAR_SPEED * m_elapsed_time;// * (2 - m_predicted.get_speed());
	  }

	 
	protected void p_predict()
	{
		if (!game_options.freeze_cpu_cars)
		{
			// computer car always accelerates
			handle_acceleration();
			apply_speed();
			m_guider.doit(angle_variation(),m_elapsed_time);
		}
		
		physical_car_behaviour(true);
	}
	
	public ComputerCar(CarBodyView body_view,
			CarShadowView shadow_view,
			ExplosionView explosion,
			WeaponFactory weapons,
			Driver driver,
			int position)  
	{	
		super(body_view,shadow_view,explosion,weapons,driver,position);
		 m_guider = new MobileGuider(this,false);
	}
}
